@@ -1,7 +1,7 @@
 ;;; cider.el --- Clojure Interactive Development Environment that Rocks -*- lexical-binding: t -*-
 
 ;; Copyright © 2012-2013 Tim King, Phil Hagelberg, Bozhidar Batsov
-;; Copyright © 2013-2017 Bozhidar Batsov, Artur Malabarba and CIDER contributors
+;; Copyright © 2013-2018 Bozhidar Batsov, Artur Malabarba and CIDER contributors
 ;;
 ;; Author: Tim King <kingtim@gmail.com>
 ;;         Phil Hagelberg <technomancy@gmail.com>
@@ -11,7 +11,7 @@
 ;;         Steve Purcell <steve@sanityinc.com>
 ;; Maintainer: Bozhidar Batsov <bozhidar@batsov.com>
 ;; URL: http://www.github.com/clojure-emacs/cider
-;; Version: 0.16.0
+;; Version: 0.17.0
 ;; Package-Requires: ((emacs "24.4") (clojure-mode "5.6.0") (pkg-info "0.4") (queue "0.1.1") (spinner "1.7") (seq "2.16"))
 ;; Keywords: languages, clojure, cider
 
@@ -60,7 +60,7 @@
   :prefix "cider-"
   :group 'applications
   :link '(url-link :tag "GitHub" "https://github.com/clojure-emacs/cider")
-  :link '(url-link :tag "Online Manual" "https://cider.readthedocs.org")
+  :link '(url-link :tag "Online Manual" "https://cider.readthedocs.io")
   :link '(emacs-commentary-link :tag "Commentary" "cider"))
 
 (defcustom cider-prompt-for-project-on-connect 'when-needed
@@ -90,12 +90,12 @@ project inference will take place."
 
 (require 'seq)
 
-(defconst cider-version "0.16.0"
+(defconst cider-version "0.17.0"
   "Fallback version used when it cannot be extracted automatically.
 Normally it won't be used, unless `pkg-info' fails to extract the
 version from the CIDER package or library.")
 
-(defconst cider-codename "Riga"
+(defconst cider-codename "Andalucía"
   "Codename used to denote stable releases.")
 
 (defcustom cider-lein-command
@@ -141,6 +141,56 @@ version from the CIDER package or library.")
   :safe #'stringp
   :package-version '(cider . "0.9.0"))
 
+(defcustom cider-clojure-cli-command
+  "clojure"
+  "The command used to execute clojure with tools.deps (requires Clojure 1.9+)."
+  :type 'string
+  :group 'cider
+  :package-version '(cider . "0.17.0"))
+
+(defcustom cider-clojure-cli-global-options
+  nil
+  "Command line options used to execute clojure with tools.deps."
+  :type 'string
+  :group 'cider
+  :safe #'stringp
+  :package-version '(cider . "0.17.0"))
+
+(defcustom cider-clojure-cli-parameters
+  "-e '(require (quote cider-nrepl.main)) (cider-nrepl.main/init %s)'"
+  "Params passed to clojure to start an nREPL server via `cider-jack-in'.
+This is evaluated using `format', with the first argument being the Clojure
+vector of middleware variables as a string."
+  :type 'string
+  :group 'cider
+  :safe #'stringp
+  :package-version '(cider . "0.17.0"))
+
+(defcustom cider-shadow-cljs-command
+  "npx shadow-cljs"
+  "The command used to execute shadow-cljs.
+
+By default we favor the project-specific shadow-cljs over the system-wide."
+  :type 'string
+  :group 'cider
+  :package-version '(cider . "0.17.0"))
+
+(defcustom cider-shadow-cljs-global-options
+  ""
+  "Command line options used to execute shadow-cljs (e.g.: -v for verbose mode)."
+  :type 'string
+  :group 'cider
+  :safe #'stringp
+  :package-version '(cider . "0.17.0"))
+
+(defcustom cider-shadow-cljs-parameters
+  "server"
+  "Params passed to shadow-cljs to start an nREPL server via `cider-jack-in'."
+  :type 'string
+  :group 'cider
+  :safe #'stringp
+  :package-version '(cider . "0.17.0"))
+
 (defcustom cider-gradle-command
   "gradle"
   "The command used to execute Gradle."
@@ -164,11 +214,12 @@ version from the CIDER package or library.")
   :safe #'stringp
   :package-version '(cider . "0.10.0"))
 
-(defcustom cider-default-repl-command
-  "lein"
+(defcustom cider-default-repl-command "clojure-cli"
   "The default command and parameters to use when connecting to nREPL.
 This value will only be consulted when no identifying file types, i.e.
-project.clj for leiningen or build.boot for boot, could be found."
+project.clj for leiningen or build.boot for boot, could be found.
+
+As tools.deps is bundled with Clojure itself, it's the default REPL command."
   :type 'string
   :group 'cider
   :safe #'stringp
@@ -185,6 +236,8 @@ cannot decide which of many build systems to use and will never override a
 command when there is no ambiguity."
   :type '(choice (const "lein")
                  (const "boot")
+                 (const "clojure-cli")
+                 (const "shadow-cljs")
                  (const "gradle")
                  (const :tag "Always ask" nil))
   :group 'cider
@@ -226,14 +279,14 @@ This variable is used by `cider-connect'."
   :package-version '(cider . "0.9.0"))
 
 (defcustom cider-auto-mode t
-  "When non-nil, automatically enable `cider-mode' for all Clojure buffers."
+  "When non-nil, automatically enable cider mode for all Clojure buffers."
   :type 'boolean
   :package-version '(cider . "0.9.0"))
 
 (defcustom cider-inject-dependencies-at-jack-in t
   "When nil, do not inject repl dependencies (most likely nREPL middlewares) at `cider-jack-in' time."
   :type 'boolean
-  :safe #'stringp
+  :safe #'booleanp
   :version '(cider . "0.11.0"))
 
 (defcustom cider-offer-to-open-cljs-app-in-browser t
@@ -265,6 +318,8 @@ Sub-match 1 must be the project path.")
   (pcase project-type
     ("lein" cider-lein-command)
     ("boot" cider-boot-command)
+    ("clojure-cli" cider-clojure-cli-command)
+    ("shadow-cljs" cider-shadow-cljs-command)
     ("gradle" cider-gradle-command)
     (_ (user-error "Unsupported project type `%s'" project-type))))
 
@@ -273,9 +328,13 @@ Sub-match 1 must be the project path.")
 Throws an error if PROJECT-TYPE is unknown.  Known types are
 \"lein\", \"boot\", and \"gradle\"."
   (pcase project-type
-    ("lein" (cider--lein-resolve-command))
-    ("boot" (cider--boot-resolve-command))
-    ("gradle" (cider--gradle-resolve-command))
+    ("lein" (cider--resolve-command cider-lein-command))
+    ("boot" (cider--resolve-command cider-boot-command))
+    ("clojure-cli" (cider--resolve-command cider-clojure-cli-command))
+    ;; here we have to account for the possibility that the command is either
+    ;; "npx shadow-cljs" or just "shadow-cljs"
+    ("shadow-cljs" (cider--resolve-command (car (split-string cider-shadow-cljs-command))))
+    ("gradle" (cider--resolve-command cider-gradle-command))
     (_ (user-error "Unsupported project type `%s'" project-type))))
 
 (defun cider-jack-in-global-options (project-type)
@@ -283,6 +342,8 @@ Throws an error if PROJECT-TYPE is unknown.  Known types are
   (pcase project-type
     ("lein" cider-lein-global-options)
     ("boot" cider-boot-global-options)
+    ("clojure-cli" cider-clojure-cli-global-options)
+    ("shadow-cljs" cider-shadow-cljs-global-options)
     ("gradle" cider-gradle-global-options)
     (_ (user-error "Unsupported project type `%s'" project-type))))
 
@@ -291,6 +352,15 @@ Throws an error if PROJECT-TYPE is unknown.  Known types are
   (pcase project-type
     ("lein" cider-lein-parameters)
     ("boot" cider-boot-parameters)
+    ("clojure-cli" (format cider-clojure-cli-parameters
+                           (concat
+                            "["
+                            (mapconcat
+                             (apply-partially #'format "\"%s\"")
+                             (cider-jack-in-normalized-nrepl-middlewares)
+                             ", ")
+                            "]")))
+    ("shadow-cljs" cider-shadow-cljs-parameters)
     ("gradle" cider-gradle-parameters)
     (_ (user-error "Unsupported project type `%s'" project-type))))
 
@@ -328,17 +398,60 @@ specifying the artifact ID, and the second element the version number."
                        (string :tag "Version"))))
 
 (defvar cider-jack-in-lein-plugins nil
-  "List of Leiningen plugins where elements are lists of artifact name and version.")
+  "List of Leiningen plugins to be injected at jack-in.
+Each element is a list of artifact name and version, followed optionally by
+keyword arguments.  The only keyword argument currently accepted is
+`:predicate', which should be given a function that takes the list (name,
+version, and keyword arguments) and returns non-nil to indicate that the
+plugin should actually be injected.  (This is useful primarily for packages
+that extend CIDER, not for users.  For example, a refactoring package might
+want to inject some middleware only when within a project context.)")
 (put 'cider-jack-in-lein-plugins 'risky-local-variable t)
 (cider-add-to-alist 'cider-jack-in-lein-plugins
                     "cider/cider-nrepl" (upcase cider-version))
 
+(defun cider-jack-in-normalized-lein-plugins ()
+  "Return a normalized list of Leiningen plugins to be injected.
+See `cider-jack-in-lein-plugins' for the format, except that the list
+returned by this function does not include keyword arguments."
+  (thread-last cider-jack-in-lein-plugins
+    (seq-filter
+     (lambda (spec)
+       (if-let* ((pred (plist-get (seq-drop spec 2) :predicate)))
+           (funcall pred spec)
+         t)))
+    (mapcar
+     (lambda (spec)
+       (seq-take spec 2)))))
+
 (defvar cider-jack-in-nrepl-middlewares nil
   "List of Clojure variable names.
-Each of these Clojure variables should hold a vector of nREPL middlewares.")
+Each of these Clojure variables should hold a vector of nREPL middlewares.
+Instead of a string, an element can be a list containing a string followed
+by optional keyword arguments.  The only keyword argument currently
+accepted is `:predicate', which should be given a function that takes the
+list (string and keyword arguments) and returns non-nil to indicate that
+the middlewares should actually be injected.")
 (put 'cider-jack-in-nrepl-middlewares 'risky-local-variable t)
 (add-to-list 'cider-jack-in-nrepl-middlewares "cider.nrepl/cider-middleware")
 
+(defun cider-jack-in-normalized-nrepl-middlewares ()
+  "Return a normalized list of middleware variable names.
+See `cider-jack-in-nrepl-middlewares' for the format, except that the list
+returned by this function only contains strings."
+  (thread-last cider-jack-in-nrepl-middlewares
+    (seq-filter
+     (lambda (spec)
+       (or (not (listp spec))
+           (if-let* ((pred (plist-get (cdr spec) :predicate)))
+               (funcall pred spec)
+             t))))
+    (mapcar
+     (lambda (spec)
+       (if (listp spec)
+           (car spec)
+         spec)))))
+
 (defun cider--list-as-boot-artifact (list)
   "Return a boot artifact string described by the elements of LIST.
 LIST should have the form (ARTIFACT-NAME ARTIFACT-VERSION).  The returned
@@ -367,7 +480,7 @@ PLUGINS and MIDDLEWARES.  PARAMS and MIDDLEWARES are passed on to
  are passed on to `cider-boot-dependencies`."
   (concat global-opts
           (unless (seq-empty-p global-opts) " ")
-          "-i \"(require 'cider.tasks)\" " ;; Note the white space at the end here
+          "-i \"(require 'cider.tasks)\" " ;; Note the space at the end here
           (cider-boot-dependencies (append dependencies plugins))
           (cider-boot-middleware-task params middlewares)))
 
@@ -385,7 +498,7 @@ string is quoted for passing as argument to an inferior shell."
   (shell-quote-argument (format "[%s %S%s]" (car list) (cadr list) (cider--lein-artifact-exclusions exclusions))))
 
 (defun cider-lein-jack-in-dependencies (global-opts params dependencies dependencies-exclusions lein-plugins)
-    "Create lein jack-in dependencies.
+  "Create lein jack-in dependencies.
 Does so by concatenating GLOBAL-OPTS, DEPENDENCIES, with DEPENDENCIES-EXCLUSIONS
 removed, LEIN-PLUGINS, and finally PARAMS."
   (concat
@@ -405,6 +518,33 @@ removed, LEIN-PLUGINS, and finally PARAMS."
    " -- "
    params))
 
+(defun cider-clojure-cli-jack-in-dependencies (global-opts params dependencies)
+  "Create Clojure tools.deps jack-in dependencies.
+Does so by concatenating GLOBAL-OPTS, DEPENDENCIES finally PARAMS."
+  (let ((dependencies (append dependencies cider-jack-in-lein-plugins)))
+    (concat
+     global-opts
+     (unless (seq-empty-p global-opts) " ")
+     "-Sdeps '{:deps {"
+     (mapconcat #'identity
+                (seq-map (lambda (dep) (format "%s {:mvn/version \"%s\"}" (car dep) (cadr dep)))  dependencies)
+                " ")
+     "}}' "
+     params)))
+
+(defun cider-shadow-cljs-jack-in-dependencies (global-opts params dependencies)
+  "Create shadow-cljs jack-in deps.
+Does so by concatenating GLOBAL-OPTS, DEPENDENCIES finally PARAMS."
+  (let ((dependencies (append dependencies cider-jack-in-lein-plugins)))
+    (concat
+     global-opts
+     (unless (seq-empty-p global-opts) " ")
+     (mapconcat #'identity
+                (seq-map (lambda (dep) (format "-d %s:%s" (car dep) (cadr dep)))  dependencies)
+                " ")
+     " "
+     params)))
+
 (defun cider-add-clojure-dependencies-maybe (dependencies)
   "Return DEPENDENCIES with an added Clojure dependency if requested.
 
@@ -437,14 +577,24 @@ dependencies."
              (cider-add-clojure-dependencies-maybe
               cider-jack-in-dependencies)
              cider-jack-in-dependencies-exclusions
-             cider-jack-in-lein-plugins))
+             (cider-jack-in-normalized-lein-plugins)))
     ("boot" (cider-boot-jack-in-dependencies
              global-opts
              params
              (cider-add-clojure-dependencies-maybe
               cider-jack-in-dependencies)
-             cider-jack-in-lein-plugins
-             cider-jack-in-nrepl-middlewares))
+             (cider-jack-in-normalized-lein-plugins)
+             (cider-jack-in-normalized-nrepl-middlewares)))
+    ("clojure-cli" (cider-clojure-cli-jack-in-dependencies
+                    global-opts
+                    params
+                    (cider-add-clojure-dependencies-maybe
+                     cider-jack-in-dependencies)))
+    ("shadow-cljs" (cider-shadow-cljs-jack-in-dependencies
+                    global-opts
+                    params
+                    (cider-add-clojure-dependencies-maybe
+                     cider-jack-in-dependencies)))
     ("gradle" (concat
                global-opts
                (unless (seq-empty-p global-opts) " ")
@@ -453,58 +603,165 @@ dependencies."
 
 
 ;;; ClojureScript REPL creation
-(defconst cider--cljs-repl-types
-  '(("(cemerick.piggieback/cljs-repl (cljs.repl.rhino/repl-env))"
-     "Rhino" "")
-    ("(do (require 'figwheel-sidecar.repl-api) (figwheel-sidecar.repl-api/start-figwheel!) (figwheel-sidecar.repl-api/cljs-repl))"
-     "Figwheel-sidecar" " (add figwheel-sidecar to your plugins)")
-    ("(do (require 'cljs.repl.node) (cemerick.piggieback/cljs-repl (cljs.repl.node/repl-env)))"
-     "Node" " (requires NodeJS to be installed)")
-    ("(do (require 'weasel.repl.websocket) (cemerick.piggieback/cljs-repl (weasel.repl.websocket/repl-env :ip \"127.0.0.1\" :port 9001)))"
-     "Weasel" " (see http://cider.readthedocs.io/en/latest/up_and_running/#browser-connected-clojurescript-repl)")
-    ("(do (require 'adzerk.boot-cljs-repl) (adzerk.boot-cljs-repl/start-repl))"
-     "Boot-cljs-repl" " (see https://github.com/adzerk-oss/boot-cljs-repl/blob/master/README.md")))
 
-(defcustom cider-cljs-lein-repl "(cemerick.piggieback/cljs-repl (cljs.repl.rhino/repl-env))"
-  "Clojure form that returns a ClojureScript REPL environment.
-This is only used in lein projects.  It is evaluated in a Clojure REPL and
-it should start a ClojureScript REPL."
-  :type `(choice ,@(seq-map (lambda (x) `(const :tag ,(apply #'concat (cdr x)) ,(car x)))
-                            cider--cljs-repl-types)
-                 (string :tag "Custom"))
-  :safe (lambda (x) (assoc x cider--cljs-repl-types))
-  :package-version '(cider . "0.11.0")
-  :group 'cider)
+(defcustom cider-check-cljs-repl-requirements t
+  "When non-nil will run the requirement checks for the different cljs repls.
 
-(defcustom cider-cljs-boot-repl "(do (require 'adzerk.boot-cljs-repl) (adzerk.boot-cljs-repl/start-repl))"
-  "Clojure form that returns a ClojureScript REPL environment.
-This is only used in boot projects.  It is evaluated in a Clojure REPL and
-it should start a ClojureScript REPL."
-  :type `(choice ,@(seq-map (lambda (x) `(const :tag ,(apply #'concat (cdr x)) ,(car x)))
-                            cider--cljs-repl-types)
-                 (string :tag "Custom"))
-  :safe (lambda (x) (assoc x cider--cljs-repl-types))
-  :package-version '(cider . "0.15.0")
-  :group 'cider)
+Generally you should not disable this unless you run into some faulty check."
+  :type 'boolean
+  :safe #'booleanp
+  :version '(cider . "0.17.0"))
 
-(defcustom cider-cljs-gradle-repl "(cemerick.piggieback/cljs-repl (cljs.repl.rhino/repl-env))"
-  "Clojure form that returns a ClojureScript REPL environment.
-This is only used in gradle projects.  It is evaluated in a Clojure REPL and
-it should start a ClojureScript REPL."
-  :type `(choice ,@(seq-map (lambda (x) `(const :tag ,(apply #'concat (cdr x)) ,(car x)))
-                            cider--cljs-repl-types)
-                 (string :tag "Custom"))
-  :safe (lambda (x) (assoc x cider--cljs-repl-types))
-  :package-version '(cider . "0.15.0")
-  :group 'cider)
+(defun cider-verify-clojurescript-is-present ()
+  "Check whether ClojureScript is present."
+  (unless (cider-library-present-p "clojurescript")
+    (user-error "ClojureScript is not available.  See http://cider.readthedocs.io/en/latest/clojurescript for details")))
 
-(defun cider-cljs-repl-form (project-type)
-  "Return a Clojure form returning a ClojureScript REPL environment based on PROJECT-TYPE."
-  (pcase project-type
-    ("lein" cider-cljs-lein-repl)
-    ("boot" cider-cljs-boot-repl)
-    ("gradle" cider-cljs-gradle-repl)
-    (_ (error "Unsupported project type `%s'" project-type))))
+(defun cider-verify-piggieback-is-present ()
+  "Check whether the piggieback middleware is present."
+  (unless (cider-library-present-p "piggieback")
+    (user-error "Piggieback is not available.  See http://cider.readthedocs.io/en/latest/clojurescript for details")))
+
+(defun cider-check-nashorn-requirements ()
+  "Check whether we can start a Nashorn ClojureScript REPL."
+  (cider-verify-piggieback-is-present)
+  (when (string-prefix-p "1.7" (cider--java-version))
+    (user-error "Nashorn is supported only on Java 8 or newer")))
+
+(defun cider-check-node-requirements ()
+  "Check whether we can start a Node ClojureScript REPL."
+  (cider-verify-piggieback-is-present)
+  (unless (executable-find "node")
+    (user-error "Node.js is not present on the exec-path.  Make sure you've installed it and your exec-path is properly set")))
+
+(defun cider-check-figwheel-requirements ()
+  "Check whether we can start a Figwheel ClojureScript REPL."
+  (cider-verify-piggieback-is-present)
+  (unless (cider-library-present-p "figwheel-sidecar")
+    (user-error "Figwheel-sidecar is not available.  Please check http://cider.readthedocs.io/en/latest/clojurescript")))
+
+(defun cider-check-weasel-requirements ()
+  "Check whether we can start a Weasel ClojureScript REPL."
+  (cider-verify-piggieback-is-present)
+  (unless (cider-library-present-p "weasel")
+    (user-error "Weasel in not available.  Please check http://cider.readthedocs.io/en/latest/clojurescript/#browser-connected-clojurescript-repl")))
+
+(defun cider-check-boot-requirements ()
+  "Check whether we can start a Boot ClojureScript REPL."
+  (cider-verify-piggieback-is-present)
+  (unless (cider-library-present-p "boot-cljs-repl")
+    (user-error "The Boot ClojureScript REPL is not available.  Please check https://github.com/adzerk-oss/boot-cljs-repl/blob/master/README.md")))
+
+(defun cider-check-shadow-cljs-requirements ()
+  "Check whether we can start a shadow-cljs REPL."
+  (unless (cider-library-present-p "shadow-cljs")
+    (user-error "The shadow-cljs ClojureScript REPL is not available")))
+
+(defun cider-shadow-cljs-init-form ()
+  "Generate the init form for a shadow-cljs REPL.
+
+We have to prompt the user to select a build, that's why
+this is a command, not just a string."
+  (let ((form "(do (require '[shadow.cljs.devtools.api :as shadow]) (shadow/watch :%s) (shadow/nrepl-select :%s))")
+        (build (string-remove-prefix ":" (read-from-minibuffer "Select shadow-cljs build: "))))
+    (format form build build)))
+
+(defun cider-custom-cljs-repl-init-form ()
+  "Prompt for a form that would start a ClojureScript REPL.
+
+The supplied string will be wrapped in a do form if needed."
+  (let ((form (read-from-minibuffer "Please, provide a form to start a ClojureScript REPL: ")))
+    ;; TODO: We should probably make this more robust (e.g. by using a regexp or
+    ;; parsing the form).
+    (if (string-prefix-p "(do" form)
+        form
+      (format "(do %s)" form))))
+
+(defvar cider-cljs-repl-types
+  '((nashorn "(cider.piggieback/cljs-repl (cljs.repl.nashorn/repl-env))"
+             cider-check-nashorn-requirements)
+    (figwheel "(do (require 'figwheel-sidecar.repl-api) (figwheel-sidecar.repl-api/start-figwheel!) (figwheel-sidecar.repl-api/cljs-repl))"
+              cider-check-figwheel-requirements)
+    (node "(do (require 'cljs.repl.node) (cider.piggieback/cljs-repl (cljs.repl.node/repl-env)))"
+          cider-check-node-requirements)
+    (weasel "(do (require 'weasel.repl.websocket) (cider.piggieback/cljs-repl (weasel.repl.websocket/repl-env :ip \"127.0.0.1\" :port 9001)))"
+            cider-check-weasel-requirements)
+    (boot "(do (require 'adzerk.boot-cljs-repl) (adzerk.boot-cljs-repl/start-repl))"
+          cider-check-boot-requirements)
+    (shadow cider-shadow-cljs-init-form cider-check-shadow-cljs-requirements)
+    (custom cider-custom-cljs-repl-init-form nil))
+  "A list of supported ClojureScript REPLs.
+
+For each one we have its name, the form we need to evaluate in a Clojure
+REPL to start the ClojureScript REPL and functions to very their requirements.
+
+The form should be either a string or a function producing a string.")
+
+(defun cider-register-cljs-repl-type (type init-form &optional requirements-fn)
+  "Register a new ClojureScript REPL type.
+
+Types are defined by the following:
+
+- TYPE - symbol identifier that will be used to refer to the REPL type
+- INIT-FORM - string or function (symbol) producing string
+- REQUIREMENTS-FN - function to check whether the REPL can be started.
+This param is optional.
+
+All this function does is modifying `cider-cljs-repl-types'.
+It's intended to be used in your Emacs config."
+  (unless (symbolp type)
+    (user-error "The REPL type must be a symbol"))
+  (unless (or (stringp init-form) (symbolp init-form))
+    (user-error "The init form must be a string or a symbol referring to a function"))
+  (unless (or (null requirements-fn) (symbolp requirements-fn))
+    (user-error "The requirements-fn must be a symbol referring to a function"))
+  (add-to-list 'cider-cljs-repl-types (list type init-form requirements-fn)))
+
+(defcustom cider-default-cljs-repl nil
+  "The default ClojureScript REPL to start.
+
+This affects commands like `cider-jack-in-clojurescript'.  Generally it's
+intended to be set via .dir-locals.el for individual projects, as its
+relatively unlikely you'd like to use the same type of REPL in each project
+you're working on."
+  :type '(choice (const :tag "Nashorn"  nashorn)
+                 (const :tag "Figwheel" figwheel)
+                 (const :tag "Node"     node)
+                 (const :tag "Weasel"   weasel)
+                 (const :tag "Boot"     boot)
+                 (const :tag "Shadow"   shadow)
+                 (const :tag "Custom"   custom))
+  :group 'cider
+  :safe #'symbolp
+  :package-version '(cider . "0.17.0"))
+
+(make-obsolete-variable 'cider-cljs-lein-repl 'cider-default-cljs-repl "0.17")
+(make-obsolete-variable 'cider-cljs-boot-repl 'cider-default-cljs-repl "0.17")
+(make-obsolete-variable 'cider-cljs-gradle-repl 'cider-default-cljs-repl "0.17")
+
+(defun cider-select-cljs-repl ()
+  "Select the ClojureScript REPL to use with `cider-jack-in-clojurescript'."
+  (let ((repl-types (mapcar #'car cider-cljs-repl-types)))
+    (intern (completing-read "Select ClojureScript REPL type: " repl-types))))
+
+(defun cider-cljs-repl-form (repl-type)
+  "Get the cljs REPL form for REPL-TYPE."
+  (let ((repl-form (cadr (seq-find
+                          (lambda (entry)
+                            (eq (car entry) repl-type))
+                          cider-cljs-repl-types))))
+    ;; repl-form can be either a string or a function producing a string
+    (if (symbolp repl-form)
+        (funcall repl-form)
+      repl-form)))
+
+(defun cider-verify-cljs-repl-requirements (repl-type)
+  "Verify that the requirements for REPL-TYPE are met."
+  (when-let* ((fun (nth 2 (seq-find
+                           (lambda (entry)
+                             (eq (car entry) repl-type))
+                           cider-cljs-repl-types))))
+    (funcall fun)))
 
 (defun cider--offer-to-open-app-in-browser (server-buffer)
   "Look for a server address in SERVER-BUFFER and offer to open it."
@@ -513,51 +770,64 @@ it should start a ClojureScript REPL."
       (save-excursion
         (goto-char (point-min))
         (when-let* ((url (and (search-forward-regexp "http://localhost:[0-9]+" nil 'noerror)
-                             (match-string 0))))
+                              (match-string 0))))
           (when (y-or-n-p (format "Visit ‘%s’ in a browser? " url))
             (browse-url url)))))))
 
 (defun cider-create-sibling-cljs-repl (client-buffer)
   "Create a ClojureScript REPL with the same server as CLIENT-BUFFER.
 The new buffer will correspond to the same project as CLIENT-BUFFER, which
-should be the regular Clojure REPL started by the server process filter."
+should be the regular Clojure REPL started by the server process filter.
+
+Normally this would prompt for the ClojureScript REPL to start (e.g. Node,
+Figwheel, etc), unless you've set `cider-default-cljs-repl'."
   (interactive (list (cider-current-connection)))
+  ;; We can't start a ClojureScript REPL without ClojureScript
+  (when cider-check-cljs-repl-requirements
+    (cider-verify-clojurescript-is-present))
   ;; Load variables in .dir-locals.el into the server process buffer, so
-  ;; cider-cljs-*-repl can be set for each project individually.
+  ;; cider-default-cljs-repl can be set for each project individually.
   (hack-local-variables)
-  (let* ((nrepl-repl-buffer-name-template "*cider-repl CLJS%s*")
-         (nrepl-create-client-buffer-function #'cider-repl-create)
-         (nrepl-use-this-as-repl-buffer 'new)
-         (client-process-args (with-current-buffer client-buffer
-                                (unless (or nrepl-server-buffer nrepl-endpoint)
-                                  (error "This is not a REPL buffer, is there a REPL active?"))
-                                (list (car nrepl-endpoint)
-                                      (elt nrepl-endpoint 1)
-                                      (when (buffer-live-p nrepl-server-buffer)
-                                        (get-buffer-process nrepl-server-buffer)))))
-         (cljs-proc (apply #'nrepl-start-client-process client-process-args))
-         (cljs-buffer (process-buffer cljs-proc))
-         (cljs-repl-form (cider-cljs-repl-form (cider-project-type))))
-    (with-current-buffer cljs-buffer
-      ;; The new connection has now been bumped to the top, but it's still a
-      ;; Clojure REPL!  Additionally, some ClojureScript REPLs can actually take
-      ;; a while to start (some even depend on the user opening a browser).
-      ;; Meanwhile, this REPL will gladly receive requests in place of the
-      ;; original Clojure REPL.  Our solution is to bump the original REPL back
-      ;; up the list, so it takes priority on Clojure requests.
-      (cider-make-connection-default client-buffer)
-      (setq cider-repl-type "cljs")
-      (pcase (assoc cljs-repl-form cider--cljs-repl-types)
-        (`(,_ ,name ,info)
-         (message "Starting a %s REPL%s" name (or info "")))
-        (_ (message "Starting a custom ClojureScript REPL")))
-      (cider-nrepl-send-request
-       `("op" "eval"
-         "ns" ,(cider-current-ns)
-         "code" ,cljs-repl-form)
-       (cider-repl-handler (current-buffer)))
-      (when cider-offer-to-open-cljs-app-in-browser
-        (cider--offer-to-open-app-in-browser nrepl-server-buffer)))))
+  (let* ((cljs-repl-type (or cider-default-cljs-repl
+                             (cider-select-cljs-repl)))
+         (cljs-repl-form (cider-cljs-repl-form cljs-repl-type)))
+    (when cider-check-cljs-repl-requirements
+      (cider-verify-cljs-repl-requirements cljs-repl-type))
+    ;; if all the requirements are met we can finally proceed with starting
+    ;; the ClojureScript REPL for `cljs-repl-type'
+    (let* ((nrepl-repl-buffer-name-template "*cider-repl%s(cljs)*")
+           (nrepl-create-client-buffer-function #'cider-repl-create)
+           (nrepl-use-this-as-repl-buffer 'new)
+           (client-process-args (with-current-buffer client-buffer
+                                  (unless (or nrepl-server-buffer nrepl-endpoint)
+                                    (error "This is not a REPL buffer, is there a REPL active?"))
+                                  (list (car nrepl-endpoint)
+                                        (elt nrepl-endpoint 1)
+                                        (when (buffer-live-p nrepl-server-buffer)
+                                          (get-buffer-process nrepl-server-buffer)))))
+           (cljs-proc (apply #'nrepl-start-client-process client-process-args))
+           (cljs-buffer (process-buffer cljs-proc)))
+      (with-current-buffer cljs-buffer
+        ;; The new connection has now been bumped to the top, but it's still a
+        ;; Clojure REPL!  Additionally, some ClojureScript REPLs can actually take
+        ;; a while to start (some even depend on the user opening a browser).
+        ;; Meanwhile, this REPL will gladly receive requests in place of the
+        ;; original Clojure REPL.  Our solution is to bump the original REPL back
+        ;; up the list, so it takes priority on Clojure requests.
+        (cider-make-connection-default client-buffer)
+        (cider-repl-set-type "cljs")
+        (pcase cider-cljs-repl-types
+          (`(,name ,_ ,info)
+           (message "Starting a %s REPL%s" name (or info ""))))
+        ;; So far we have just another Clojure REPL.  It's time to convert it
+        ;; to a ClojureScript REPL with a magic incantation.
+        (cider-nrepl-send-request
+         `("op" "eval"
+           "ns" ,(cider-current-ns)
+           "code" ,cljs-repl-form)
+         (cider-repl-handler (current-buffer)))
+        (when cider-offer-to-open-cljs-app-in-browser
+          (cider--offer-to-open-app-in-browser nrepl-server-buffer))))))
 
 (defun cider--select-zombie-buffer (repl-buffers)
   "Return a zombie buffer from REPL-BUFFERS, or nil if none exists."
@@ -623,7 +893,7 @@ own buffer."
                            (cider-inject-jack-in-dependencies command-global-opts params project-type)
                          params))
 
-               (cmd (format "%s %s" command-resolved params)))
+               (cmd (format "%s %s" command params)))
           (if (or project-dir cider-allow-jack-in-without-project)
               (progn
                 (when (or project-dir
@@ -648,6 +918,9 @@ start the server."
   (interactive "P")
   (cider-jack-in prompt-project 'cljs-too))
 
+;;;###autoload
+(defalias 'cider-jack-in-cljs #'cider-jack-in-clojurescript)
+
 ;;;###autoload
 (defun cider-connect (host port &optional project-dir)
   "Connect to an nREPL server identified by HOST and PORT.
@@ -671,12 +944,28 @@ gets associated with it."
            ;; associate if we're in a project, prompt otherwise
            ((eq cider-prompt-for-project-on-connect 'when-needed) (cider-assoc-project-with-connection project-dir conn))
            ;; always prompt
-           (t (cider-assoc-project-with-connection nil conn))))))))
+           (t (cider-assoc-project-with-connection nil conn)))))
+      conn)))
+
+;;;###autoload
+(defun cider-connect-clojurescript ()
+  "Connect to a ClojureScript REPL.
+
+It just delegates pretty much everything to `cider-connect' and just sets
+the appropriate REPL type in the end."
+  (interactive)
+  (when-let* ((conn (call-interactively #'cider-connect)))
+    (with-current-buffer conn
+      (cider-repl-set-type "cljs"))))
+
+;;;###autoload
+(defalias 'cider-connect-cljs #'cider-connect-clojurescript)
 
 (defun cider-current-host ()
   "Retrieve the current host."
   (if (and (stringp buffer-file-name)
-           (file-remote-p buffer-file-name))
+           (file-remote-p buffer-file-name)
+           (boundp 'tramp-current-host))
       tramp-current-host
     "localhost"))
 
@@ -780,6 +1069,8 @@ Use `cider-ps-running-nrepls-command' and `cider-ps-running-nrepl-path-regexp-li
   (let* ((default-directory (clojure-project-dir (cider-current-dir)))
          (build-files '(("lein" . "project.clj")
                         ("boot" . "build.boot")
+                        ("clojure-cli" . "deps.edn")
+                        ("shadow-cljs" . "shadow-cljs.edn")
                         ("gradle" . "build.gradle"))))
     (delq nil
           (mapcar (lambda (candidate)
@@ -788,8 +1079,8 @@ Use `cider-ps-running-nrepls-command' and `cider-ps-running-nrepl-path-regexp-li
                   build-files))))
 
 (defun cider-project-type ()
-  "Determine the type, either leiningen, boot or gradle, of the current project.
-If more than one project file types are present, check for a preferred
+  "Determine the type of the current project.
+When multiple project file markers are present, check for a preferred
 build tool in `cider-preferred-build-tool', otherwise prompt the user to
 choose."
   (let* ((choices (cider--identify-buildtools-present))
@@ -806,34 +1097,14 @@ choose."
           (t cider-default-repl-command))))
 
 
-;; TODO: Implement a check for `cider-lein-command' over tramp
-(defun cider--lein-resolve-command ()
-  "Find `cider-lein-command' on `exec-path' if possible, or return nil.
+;; TODO: Implement a check for command presence over tramp
+(defun cider--resolve-command (command)
+  "Find COMMAND on `exec-path' if possible, or return nil.
 
 In case `default-directory' is non-local we assume the command is available."
-  (when-let* ((command (or (and (file-remote-p default-directory) cider-lein-command)
-                           (executable-find cider-lein-command)
-                           (executable-find (concat cider-lein-command ".bat")))))
-    (shell-quote-argument command)))
-
-;; TODO: Implement a check for `cider-boot-command' over tramp
-(defun cider--boot-resolve-command ()
-  "Find `cider-boot-command' on `exec-path' if possible, or return nil.
-
-In case `default-directory' is non-local we assume the command is available."
-  (when-let* ((command (or (and (file-remote-p default-directory) cider-boot-command)
-                           (executable-find cider-boot-command)
-                           (executable-find (concat cider-boot-command ".exe")))))
-    (shell-quote-argument command)))
-
-;; TODO: Implement a check for `cider-gradle-command' over tramp
-(defun cider--gradle-resolve-command ()
-  "Find `cider-gradle-command' on `exec-path' if possible, or return nil.
-
-In case `default-directory' is non-local we assume the command is available."
-  (when-let* ((command (or (and (file-remote-p default-directory) cider-gradle-command)
-                           (executable-find cider-gradle-command)
-                           (executable-find (concat cider-gradle-command ".exe")))))
+  (when-let* ((command (or (and (file-remote-p default-directory) command)
+                           (executable-find command)
+                           (executable-find (concat command ".bat")))))
     (shell-quote-argument command)))
 
 
@@ -874,6 +1145,15 @@ message in the REPL area."
                                  "CIDER's version (%s) does not match cider-nrepl's version (%s). Things will break!"
                                  cider-version middleware-version))))
 
+(defcustom cider-redirect-server-output-to-repl  t
+  "Controls whether nREPL server output would be redirected to the REPL.
+When non-nil the output would end up in both the nrepl-server buffer (when
+available) and the matching REPL buffer."
+  :type 'boolean
+  :group 'cider
+  :safe #'booleanp
+  :package-version '(cider . "0.17.0"))
+
 (defun cider--subscribe-repl-to-server-out ()
   "Subscribe to the nREPL server's *out*."
   (cider-nrepl-send-request '("op" "out-subscribe")
@@ -893,7 +1173,8 @@ buffer."
     (cider--check-required-nrepl-version)
     (cider--check-clojure-version-supported)
     (cider--check-middleware-compatibility)
-    (cider--subscribe-repl-to-server-out)
+    (when cider-redirect-server-output-to-repl
+      (cider--subscribe-repl-to-server-out))
     (when cider-auto-mode
       (cider-enable-on-existing-clojure-buffers))
     ;; Middleware on cider-nrepl's side is deferred until first usage, but
@@ -917,7 +1198,8 @@ process buffer."
   '(progn
      (define-key clojure-mode-map (kbd "C-c M-j") #'cider-jack-in)
      (define-key clojure-mode-map (kbd "C-c M-J") #'cider-jack-in-clojurescript)
-     (define-key clojure-mode-map (kbd "C-c M-c") #'cider-connect)))
+     (define-key clojure-mode-map (kbd "C-c M-c") #'cider-connect)
+     (define-key clojure-mode-map (kbd "C-c M-C") #'cider-connect-clojurescript)))
 
 (provide 'cider)
 
